﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Resources;
using System.Threading;
using System.Reflection;
using System.IO;
using System.Globalization;
using System.Diagnostics;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;
using RootPro.RootProCAD.UI;

//////////////////////////////////////////////////////////////////
// 分割割付のアドイン
// 2 つの図形の間に、等間隔に図形を作成します。
//////////////////////////////////////////////////////////////////

namespace RcAddInTriangleCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        string IsoscelesTriangleCommandName = "RootPro.AddIn.IsoscelesTriangleCommand";     // 二等辺三角形
        string TriangleCommandName = "RootPro.AddIn.TriangleCommand";     // 三角形
        
        private void AppAddIn_Startup(object sender, EventArgs e)
        {
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

            string[] parentMenus = { Properties.Resources.MenuName };

            CommandManager.AddMacroCommand("RcAddInTriangleCS", parentMenus, Properties.Resources.CommandName1, MacroCommandTriangle);
            CommandManager.AddMacroCommand("RcAddInIsoscelesTriangleCS", parentMenus, Properties.Resources.CommandName2, MacroCommandIsoscelesTriangle);

            // 三角形コマンドを追加する
            Command command1 = CommandManager.AddCommand(TriangleCommandName, Properties.Resources.CommandName1);

            // 図形パラメータ
            command1.ParameterGroups.Add(ParameterItemType.Shape, 0, Properties.Resources.ShapeParameterName);
            ShapeParameterItem shapeParameterItem = (ShapeParameterItem)command1.GetParameterItemByID(0);
            shapeParameterItem.ShapeTypeFilter = ShapeTypeFilters.Line;      // 線分のみ
            shapeParameterItem.Description = Properties.Resources.ShapeParameterDescription;

            // 辺の長さ 1
            CommandParameterGroup commandParameterGroup = command1.ParameterGroups.Add(ParameterItemType.Length, 1, Properties.Resources.Length1ParameterName);
            LengthParameterItem lengthParameterItem = (LengthParameterItem)command1.GetParameterItemByID(1);
            lengthParameterItem.Description = Properties.Resources.LengthParameterDescription;

            // 端点の角度 1
            commandParameterGroup.Items.Add(ParameterItemType.Angle, 4, Properties.Resources.BaseAngleParameter1Name);
            AngleParameterItem angleParameterItem = (AngleParameterItem)command1.GetParameterItemByID(4);
            angleParameterItem.Description = Properties.Resources.BaseAngleParameter1Description;

            // 辺の長さ 2
            command1.ParameterGroups.Add(ParameterItemType.Length, 2, Properties.Resources.Length2ParameterName);
            lengthParameterItem = (LengthParameterItem)command1.GetParameterItemByID(2);
            lengthParameterItem.Description = Properties.Resources.LengthParameterDescription;

            // 方向点
            command1.ParameterGroups.Add(ParameterItemType.Point, 3, Properties.Resources.DirectionPointParameterName);
            PointParameterItem pointParameterItem = (PointParameterItem)command1.GetParameterItemByID(3);
            pointParameterItem.Description = Properties.Resources.DirectionPointParameterDescription;

            //開始イベントと終了イベントの追加
            command1.Begin += new CommandBeginEventHandler(command_Begin1);
            command1.End += new EventHandler(command_End1);

            // 二等辺三角形コマンドを追加する
            Command command2 = CommandManager.AddCommand(IsoscelesTriangleCommandName, Properties.Resources.CommandName2);

            // 図形パラメータ
            command2.ParameterGroups.Add(ParameterItemType.Shape, 0, Properties.Resources.ShapeParameterName);
            shapeParameterItem = (ShapeParameterItem)command2.GetParameterItemByID(0);
            shapeParameterItem.ShapeTypeFilter = ShapeTypeFilters.Line;      // 線分のみ
            shapeParameterItem.Description = Properties.Resources.ShapeParameterDescription;

            // 二辺の長さ
            commandParameterGroup = command2.ParameterGroups.Add(ParameterItemType.Length, 1, Properties.Resources.Length2SidesParameterName);
            lengthParameterItem = (LengthParameterItem)command2.GetParameterItemByID(1);
            lengthParameterItem.Description = Properties.Resources.Length2SidesParameterDescription;
            lengthParameterItem.ValueSaveMode = ParameterValueSaveMode.InCommand;

            // 低角
            commandParameterGroup.Items.Add(ParameterItemType.Angle, 2, Properties.Resources.BaseAngleParameterName);
            angleParameterItem = (AngleParameterItem)command2.GetParameterItemByID(2);
            angleParameterItem.Description = Properties.Resources.BaseAngleParameterDescription;
            angleParameterItem.ValueSaveMode = ParameterValueSaveMode.InCommand;

            // 頂角
            commandParameterGroup.Items.Add(ParameterItemType.Angle, 3, Properties.Resources.ApexAngleParameterName);
            angleParameterItem = (AngleParameterItem)command2.GetParameterItemByID(3);
            angleParameterItem.Description = Properties.Resources.ApexAngleParameterDescription;
            angleParameterItem.ValueSaveMode = ParameterValueSaveMode.InCommand;

            // 方向点
            command2.ParameterGroups.Add(ParameterItemType.Point, 4, Properties.Resources.DirectionPointParameterName);
            pointParameterItem = (PointParameterItem)command2.GetParameterItemByID(4);
            pointParameterItem.Description = Properties.Resources.DirectionPointParameterDescription;

            //開始イベントと終了イベントの追加
            command2.Begin += new CommandBeginEventHandler(command_Begin2);
            command2.End += new EventHandler(command_End2);

        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            CommandManager.RemoveMacroCommand(MacroCommandIsoscelesTriangle);
            CommandManager.RemoveMacroCommand(MacroCommandTriangle);
            CommandManager.RemoveCommand(IsoscelesTriangleCommandName);
            CommandManager.RemoveCommand(TriangleCommandName);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// 三角形コマンド処理
        /// </summary>
        private void MacroCommandTriangle()
        {
            //コマンドの実行
            CommandManager.ExecuteCommand(TriangleCommandName);
        }
        
        /// <summary>
        /// 二等辺三角形コマンド処理
        /// </summary>
        private void MacroCommandIsoscelesTriangle()
        {
            //コマンドの実行
            CommandManager.ExecuteCommand(IsoscelesTriangleCommandName);
        }
    
        /// <summary>
        /// 三角形コマンドの開始時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_Begin1(Object sender, CommandBeginEventArgs e)
        {
            Command command = (Command)sender;

            // パラメータアイテムの変更イベントの追加
            command.ParameterGroupCurrentIndexChanged += new CommandParameterGroupCurrentIndexChangedEventHandler(command_ParameterGroupCurrentIndexChanged1);
            //パラメータ変更イベントの追加
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged1);
            //マウス移動イベントの追加
            command.MouseMove += new CommandMouseMoveEventHandler(command_MouseMove1);
        }

        /// <summary>
        /// 三角形コマンドの終了時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_End1(Object sender, EventArgs e)
        {
            Command command = (Command)sender;

            // パラメータアイテムの変更イベントの削除
            command.ParameterGroupCurrentIndexChanged -= new CommandParameterGroupCurrentIndexChangedEventHandler(command_ParameterGroupCurrentIndexChanged1);
            //パラメータ変更イベントの削除
            command.ParameterChanged -= new CommandParameterChangedEventHandler(command_ParameterChanged1);
            //マウス移動イベントの削除
            command.MouseMove -= new CommandMouseMoveEventHandler(command_MouseMove1);
        }

        /// <summary>
        /// 二等辺三角形コマンドの開始時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_Begin2(Object sender, CommandBeginEventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの追加
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged2);
            //マウス移動イベントの追加
            command.MouseMove += new CommandMouseMoveEventHandler(command_MouseMove2);
            
        }

        /// <summary>
        /// 二等辺三角形コマンドの終了時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_End2(Object sender, EventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの削除
            command.ParameterChanged -= new CommandParameterChangedEventHandler(command_ParameterChanged2);
            //マウス移動イベントの削除
            command.MouseMove -= new CommandMouseMoveEventHandler(command_MouseMove2);
        }

        /// <summary>
        /// パラメータグループのカレントのパラメータアイテムが変更されたときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterGroupCurrentIndexChanged1(Object sender, CommandParameterGroupCurrentIndexChangedEventArgs e)
        {

            Document doc = ActiveDocument;         //ドキュメントの取得

            Command command = (Command)sender;

            CommandParameterGroup parameterGroup = e.ParameterGroup;    // 変更されたパラメータグループ
            if(command.ParameterGroups.IndexOf(parameterGroup) != 1)
                return;

            // 辺の長さ→端点の角度
            if(e.OldIndex == 0 && e.NewIndex == 1)
            {
                // 辺の長さ 2 を削除
                command.ParameterGroups.RemoveAt(2);

                // 端点の角度 2 を挿入
                command.ParameterGroups.Insert(2, ParameterItemType.Angle, 5, Properties.Resources.BaseAngleParameter2Name);
                AngleParameterItem angleParameterItem = (AngleParameterItem)command.GetParameterItemByID(5);
                angleParameterItem.Description = Properties.Resources.BaseAngleParameter2Description;
            }
            // 端点の角度→辺の長さ
            else if(e.OldIndex == 1 && e.NewIndex == 0)
            {
                // 端点の角度 2 を削除
                command.ParameterGroups.RemoveAt(2);
 
                // 辺の長さ 2 を挿入
                command.ParameterGroups.Insert(2, ParameterItemType.Length, 2, Properties.Resources.Length2ParameterName);
                LengthParameterItem lengthParameterItem = (LengthParameterItem)command.GetParameterItemByID(2);
                lengthParameterItem.Description = Properties.Resources.LengthParameterDescription;
            }
        }

        /// <summary>
        /// 三角形コマンドのパラメータが変化したときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterChanged1(Object sender, CommandParameterChangedEventArgs e)
        {
            Document doc = ActiveDocument;         //ドキュメントの取得

            Command command = (Command)sender;
            int id = e.ParameterID;// 変更のあったパラメータID

            double length1 = 0.0, length2 = 0.0;
            double angle1 = 0.0, angle2 = 0.0;
            LineShape lineShape = null;

            // 辺の長さ指定の場合
            if(command.ParameterGroups[1].CurrentIndex == 0)
            {
                // 辺の長さ 1
                LengthParameterItem lengthParameterItem = (LengthParameterItem)command.GetParameterItemByID(1);
                if(lengthParameterItem != null && !lengthParameterItem.IsEmpty)
                    length1 = lengthParameterItem.Length;

                // 辺の長さ 2
                lengthParameterItem = (LengthParameterItem)command.GetParameterItemByID(2);
                if(lengthParameterItem != null && !lengthParameterItem.IsEmpty)
                    length2 = lengthParameterItem.Length;

            }
            // 角度指定の場合
            else
            {
                // 辺の長さ 1
                AngleParameterItem angleParameterItem = (AngleParameterItem)command.GetParameterItemByID(4);
                if(angleParameterItem != null && !angleParameterItem.IsEmpty)
                    angle1 = angleParameterItem.Angle;

                // 辺の長さ 2
                angleParameterItem = (AngleParameterItem)command.GetParameterItemByID(5);
                if(angleParameterItem != null && !angleParameterItem.IsEmpty)
                    angle2 = angleParameterItem.Angle;

            }

            // 図形
            ShapeParameterItem shapeParameterItem = (ShapeParameterItem)command.GetParameterItemByID(0);
            if(shapeParameterItem != null && !shapeParameterItem.IsEmpty)
            {
                lineShape = (LineShape)shapeParameterItem.Shape.Shape;
                length1 /= shapeParameterItem.Shape.Layout.ScaleX;
                length2 /= shapeParameterItem.Shape.Layout.ScaleX;
            }
            
            // パラメータの入力チェック
            // 辺の長さは 0.0以上
            if(id == 1 && length1 <= 0.0)
            {
                e.CommandState = CommandState.ParameterError;
                return;
            }
            else if(id == 2 && length2 <= 0.0)
            {
                e.CommandState = CommandState.ParameterError;
                return;
            }

            // 角度は 1.0 ～ 179.0
            else if(id == 4 && (angle1 <= 1.0 || angle1 >= 179.0))
            {
                e.CommandState = CommandState.ParameterError;
                return;
            }
            else if(id == 5 && (angle1 <= 1.0 || angle1 >= 179.0))
            {
                e.CommandState = CommandState.ParameterError;
                return;
            }
            // 角度の合計は 179.0 まで
            if(id == 4 || id == 5)
            {
                if(angle1 + angle2 > 179.0)
                {
                    e.CommandState = CommandState.ParameterError;
                    return;
                }
            }
            // 図形、辺の長さが入力済みになったときのチェック
            if((lineShape != null && length1 != 0.0 && length2 != 0.0) && (id == 0 || id == 1 || id == 2))
            {
                // 底辺の長さ
                List<double> list = new List<double>();
                list.Add(lineShape.Length);
                list.Add(length1);
                list.Add(length2);
                list.Sort();
                // 短い 2 辺が短かすぎる場合
                if(list[2] >= list[0] + list[1])
                {
                    e.CommandState = CommandState.ParameterError;
                    return;
                }
            }

            // 図形チェック
            if(lineShape == null)
                return;

            // 方向点
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetParameterItemByID(3);
            if(pointParameterItem == null || pointParameterItem.IsEmpty)
                return;

            Point2d point = pointParameterItem.Point;

            // 現在のレイアウトの座標系からワールド座標に変換
            point = ActiveDocument.CurrentLayout.TransformPointLocalToWorld(point);
            // ワールド座標から図形の座標系に変換
            point = shapeParameterItem.Shape.Layout.TransformPointWorldToLocal(point);

            // 図形か方向点が変更されたとき
            if(id == 0 || id == 3)
            {
                if(lineShape.GetPointSide(point) == PointSide.On)
                {
                    e.CommandState = CommandState.ParameterError;
                    return;
                }
            }

            // 三角形を作成
            LineShape[] lineSides; 
            // 辺の長さ指定の場合
            if(command.ParameterGroups[1].CurrentIndex == 0)
            {
                if(length1 == 0.0 || length2 == 0.0)
                {
                    e.CommandState = CommandState.Continue;
                    return;
                }

                // 二辺の長さで三角形を作成
                lineSides = CreateTriangleSides(lineShape, length1, length2, point);
            }
            // 角度指定の場合
            else
            {
                if(angle1 == 0.0 || angle2 == 0.0)
                {
                    e.CommandState = CommandState.Continue;
                    return;
                }
                // 三角形を作成
                lineSides = CreateTriangleSidesByAngle(lineShape, angle1, angle2, point);
            }

            if(lineSides.Length == 2)
            {
                // UNDO 可能な操作を開始
                doc.UndoManager.BeginUndoUnit();

                lineShape.Drawing.Shapes.Add(lineSides[0]);
                lineShape.Drawing.Shapes.Add(lineSides[1]);

                // UNDO 可能な操作を終了
                doc.UndoManager.EndUndoUnit();

                e.CommandState = CommandState.Execute;
            }
            else
            {

            }
        }

        /// <summary>
        /// 三角形コマンドでマウスが移動したときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void command_MouseMove1(Object sender, CommandMouseMoveEventArgs e)
        {
            double length1 = 0.0, length2 = 0.0;
            double angle1 = 0.0, angle2 = 0.0;
            Command command = (Command)sender;
            // Command.GetTemporaryParameterItemByID を使用してコマンドの一時的なパラメータアイテムを取得します。
            // 一時的なパラメータアイテムからは、すでにパラメータの入力が確定している場合は確定している入力値が、
            // 入力が確定していない場合はマウスの現在位置などの一時的な値が取得できます。

            // 図形の一時パラメータを取得する
            ShapeParameterItem shapeParameterItem = (ShapeParameterItem)command.GetTemporaryParameterItemByID(0);
            if(shapeParameterItem == null || shapeParameterItem.IsEmpty)
                return;
            LineShape lineShape = (LineShape)shapeParameterItem.Shape.Shape;

            // 方向点の一時パラメータを取得する
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetTemporaryParameterItemByID(3);
            if(pointParameterItem == null || pointParameterItem.IsEmpty)
                return;

            Point2d point = pointParameterItem.Point;
            // 現在のレイアウトの座標系からワールド座標に変換
            point = ActiveDocument.CurrentLayout.TransformPointLocalToWorld(point);
            // ワールド座標から図形の座標系に変換
            point = shapeParameterItem.Shape.Layout.TransformPointWorldToLocal(point);

            LineShape[] lineShapes = null;
            // 辺の長さ指定の場合
            if(command.ParameterGroups[1].CurrentIndex == 0)
            {
                // 辺の長さの一時パラメータを取得する
                LengthParameterItem lengthParameterItem = (LengthParameterItem)command.GetTemporaryParameterItemByID(1);
                if(lengthParameterItem != null && !lengthParameterItem.IsEmpty)
                {
                    length1 = lengthParameterItem.Length;
                    length1 /= shapeParameterItem.Shape.Layout.ScaleX;
                }

                // 辺の長さの一時パラメータを取得する
                lengthParameterItem = (LengthParameterItem)command.GetTemporaryParameterItemByID(2);
                if(lengthParameterItem != null && !lengthParameterItem.IsEmpty)
                {
                    length2 = lengthParameterItem.Length;
                    length2 /= shapeParameterItem.Shape.Layout.ScaleX;
                }

                if(length1 <= 0.0 || length2 <= 0.0)
                    return;

                // 二辺の線図形を取得
                lineShapes = CreateTriangleSides(lineShape, length1, length2, point);
            }
            // 角度指定の場合
            else
            {
                // 辺の長さの一時パラメータを取得する
                AngleParameterItem angleParameterItem = (AngleParameterItem)command.GetTemporaryParameterItemByID(4);
                if(angleParameterItem != null && !angleParameterItem.IsEmpty)
                    angle1 = angleParameterItem.Angle;

                // 辺の長さの一時パラメータを取得する
                angleParameterItem = (AngleParameterItem)command.GetTemporaryParameterItemByID(5);
                if(angleParameterItem != null && !angleParameterItem.IsEmpty)
                    angle2 = angleParameterItem.Angle;

                if(angle1 <= 0.0 || angle2 <= 0.0)
                    return;

                // 二辺の線図形を取得
                lineShapes = CreateTriangleSidesByAngle(lineShape, angle1, angle2, point);
            }

            try
            {
                if(lineShapes != null && lineShapes.Length == 2)
                {
                    // ラバーバンドの色を設定
                    lineShapes[0].ColorNumber = 3;
                    lineShapes[1].ColorNumber = 3;

                    // 現在の座標系に変換してラバーバンド表示
                    Shape[] lineShapes2 = shapeParameterItem.Shape.Layout.TransformShapesLocalToWorld(lineShapes);
                    Shape[] lineShapes3 = ActiveDocument.CurrentLayout.TransformShapesWorldToLocal(lineShapes2);
                    command.DrawShapeRubberBand(lineShapes3);
                }
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// 二辺の長さを指定して三角形を作成する
        /// </summary>
        /// <param name="shape"></param>
        /// <param name="length1"></param>
        /// <param name="length2"></param>
        /// <param name="point"></param>
        private LineShape[] CreateTriangleSides(LineShape shape, double length1, double length2, Point2d point)
        {
            ArrayList shapes = new ArrayList();

            // 底辺の長さ
            double baseLineLength = shape.Length;
            List<double> list = new List<double>();
            list.Add(baseLineLength);
            list.Add(length1);
            list.Add(length2);
            list.Sort();
            // 短い 2 辺が短かすぎる場合
            if(list[2] >= list[0] + list[1])
                return (LineShape[])shapes.ToArray(typeof(LineShape));

            // 方向点がどちら側かを取得
            PointSide pointSide = shape.GetPointSide(point);
            if(pointSide == PointSide.On)
                return (LineShape[])shapes.ToArray(typeof(LineShape));

            // 底辺の角度 1
            double baseAngle1 =
                Math.Acos((Math.Pow(length1, 2) + Math.Pow(baseLineLength, 2) - Math.Pow(length2, 2)) / (2.0 * length1 * baseLineLength));

            // 底辺の角度 2
            double baseAngle2 =
                Math.Acos((Math.Pow(length2, 2) + Math.Pow(baseLineLength, 2) - Math.Pow(length1, 2)) / (2.0 * length2 * baseLineLength));

            // 角度を度に変換
            baseAngle1 = Geometry.RadianToDegree(baseAngle1);
            baseAngle2 = Geometry.RadianToDegree(baseAngle2);

            return CreateTriangleSidesByAngle(shape, baseAngle1, baseAngle2, point); ;
        }

        /// <summary>
        /// 角度を指定して三角形の二辺を作成する
        /// </summary>
        /// <param name="shape"></param>
        /// <param name="angle1"></param>
        /// <param name="angle2"></param>
        /// <param name="point"></param>
        private LineShape[] CreateTriangleSidesByAngle(LineShape shape, double angle1, double angle2, Point2d point)
        {
            ArrayList shapes = new ArrayList();

            if(angle1 < 1.0 || angle2 < 1.0 || angle1 + angle2 < 1.0 || angle1 + angle2 > 179.0)
                return (LineShape[])shapes.ToArray(typeof(LineShape));

            double baseLineLength = shape.Length;

            // 方向点がどちら側かを取得
            PointSide pointSide = shape.GetPointSide(point);
            if(pointSide == PointSide.On)
                return (LineShape[])shapes.ToArray(typeof(LineShape));

            // 底辺の角度
            double baseLineAngle = Geometry.GetAngle(shape.StartPoint, shape.EndPoint);

            // 底辺の中点を通る垂直線を作成
            InfiniteLineShape verticalLine = 
                ShapeFactory.CreateInfiniteLine(Geometry.GetMiddlePoint(shape.StartPoint, shape.EndPoint), Geometry.WrapAngleTo360(baseLineAngle + 90.0));

            // 方向点がどちら側かを取得
            PointSide pointSideVertical = verticalLine.GetPointSide(point);

            // 方向点の位置で頂点の位置を入れ替える
            if(pointSide == PointSide.Right)
            {
                // 角度を - にする
                angle1 = Geometry.WrapAngleTo360(-angle1);
                angle2 = Geometry.WrapAngleTo360(-angle2);
            }
            if(pointSide == PointSide.Right ^ (pointSideVertical == PointSide.Right ^ angle1 < angle2))
            {
                // 角度を入れ替える
                double temp = angle1;
                angle1 = angle2;
                angle2 = temp;
            }

            // 始点からの角度線を作成
            InfiniteLineShape tempLine1 = ShapeFactory.CreateInfiniteLine(shape.StartPoint, Geometry.WrapAngleTo360(baseLineAngle + angle1));
            // 終点からの角度線を作成
            InfiniteLineShape tempLine2 = ShapeFactory.CreateInfiniteLine(shape.EndPoint, Geometry.WrapAngleTo360(180.0 - angle2 + baseLineAngle));

            // 二つの線分の交点を取得
            IntersectionPoint[] intersectionPoint = tempLine1.GetIntersectionPoint(tempLine2);

            if(intersectionPoint.Length == 1)
            {
                // 交点までの線分をセット
                Point2d apexPoint = intersectionPoint[0].PointOnShape1.Point;
                LineShape line1 = (LineShape)shape.Clone();
                line1.SetPoint(shape.StartPoint, apexPoint);
                shapes.Add(line1);
                LineShape line2 = (LineShape)shape.Clone();
                line2.SetPoint(shape.EndPoint, apexPoint);
                shapes.Add(line2);
            }

            return (LineShape[])shapes.ToArray(typeof(LineShape));
        }

        /// <summary>
        /// 二等辺三角形コマンドのパラメータが変化したときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_ParameterChanged2(Object sender, CommandParameterChangedEventArgs e)
        {
            Document doc = ActiveDocument;         //ドキュメントの取得

            Command command = (Command)sender;
            int id = e.ParameterID;// 変更のあったパラメータID
            ParameterItem parameterItem = command.GetParameterItemByID(id); // パラメータアイテム

            double length = 0.0, angle = 0.0;

            // 低角
            AngleParameterItem angleParameterItem = (AngleParameterItem)command.GetParameterItemByID(2);
            if(!angleParameterItem.IsEmpty && angleParameterItem.Group.CurrentItem.ID == 2)
                angle = angleParameterItem.Angle;

            // 頂角
            AngleParameterItem angleParameterItem2 = (AngleParameterItem)command.GetParameterItemByID(3);
            if(!angleParameterItem2.IsEmpty && angleParameterItem2.Group.CurrentItem.ID == 3)
                angle = (180.0 - angleParameterItem2.Angle) / 2.0;

            // 角度が入力されたときは範囲チェック
            if(id == 2 || id == 3)
            {
                if(angle < 1.0 || angle > 89.0)
                {
                    e.CommandState = CommandState.ParameterError;
                    return;
                }
            }

            // 図形
            ShapeParameterItem shapeParameterItem = (ShapeParameterItem)command.GetParameterItemByID(0);
            if(shapeParameterItem.IsEmpty)
                return;
            LineShape lineShape = (LineShape)shapeParameterItem.Shape.Shape;

            // 二辺の長さ
            LengthParameterItem lengthParameterItem = (LengthParameterItem)command.GetParameterItemByID(1);
            if(!lengthParameterItem.IsEmpty && lengthParameterItem.Group.CurrentItem.Equals(lengthParameterItem))
            {
                length = lengthParameterItem.Length;
                length /= shapeParameterItem.Shape.Layout.ScaleX;

                // パラメータチェック
                double baseLineLength = lineShape.Length;
                if(baseLineLength >= length * 2)
                {
                    e.CommandState = CommandState.ParameterError;
                    return;
                }
            }

            // 方向点
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetParameterItemByID(4);
            if(pointParameterItem.IsEmpty)
                return;

            Point2d point = pointParameterItem.Point;

            // 現在のレイアウトの座標系からワールド座標に変換
            point = ActiveDocument.CurrentLayout.TransformPointLocalToWorld(point);
            // ワールド座標から図形の座標系に変換
            point = shapeParameterItem.Shape.Layout.TransformPointWorldToLocal(point);


            if(lineShape.GetPointSide(pointParameterItem.Point) == PointSide.On)
            {
                e.CommandState = CommandState.ParameterError;
                return;
            }

            if(length == 0.0 && angle == 0.0)
            {
                e.CommandState = CommandState.Continue;
                return;
            }

            //二等辺三角形を作成
            LineShape[] lineSides = Create2Sides(lineShape, length, angle, point);

            if(lineSides.Length == 2)
            {
                // UNDO 可能な操作を開始
                doc.UndoManager.BeginUndoUnit();

                lineShape.Drawing.Shapes.Add(lineSides[0]);
                lineShape.Drawing.Shapes.Add(lineSides[1]);

                // UNDO 可能な操作を終了
                doc.UndoManager.EndUndoUnit();

                e.CommandState = CommandState.Execute;
            }
        }

        /// <summary>
        /// 二等辺三角形コマンドでマウスが移動したときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void command_MouseMove2(Object sender, CommandMouseMoveEventArgs e)
        {
            double length = 0.0, angle = 0.0;
            Command command = (Command)sender;
            // Command.GetTemporaryParameterItemByID を使用してコマンドの一時的なパラメータアイテムを取得します。
            // 一時的なパラメータアイテムからは、すでにパラメータの入力が確定している場合は確定している入力値が、
            // 入力が確定していない場合はマウスの現在位置などの一時的な値が取得できます。

            // 図形の一時パラメータを取得する
            ShapeParameterItem shapeParameterItem = (ShapeParameterItem)command.GetTemporaryParameterItemByID(0);
            if(shapeParameterItem == null || shapeParameterItem.IsEmpty)
                return;
            LineShape lineShape = (LineShape)shapeParameterItem.Shape.Shape;

            if(command.ParameterGroups[1].CurrentIndex == 0)
            {
                // 長さの一時パラメータを取得する
                LengthParameterItem lengthParameterItem = (LengthParameterItem)command.GetTemporaryParameterItemByID(1);
                if (lengthParameterItem != null && !lengthParameterItem.IsEmpty)
                {
                    length = lengthParameterItem.Length;
                    length /= shapeParameterItem.Shape.Layout.ScaleX;
                }
            }
            else if(command.ParameterGroups[1].CurrentIndex == 1)
            {
                // 低角の一時パラメータを取得する
                AngleParameterItem angleParameterItem = (AngleParameterItem)command.GetTemporaryParameterItemByID(2);
                if (angleParameterItem != null && !angleParameterItem.IsEmpty && angleParameterItem.Group.CurrentItem.ID == 2)
                    angle = angleParameterItem.Angle;
            }
            else if(command.ParameterGroups[1].CurrentIndex == 2)
            {
                // 頂角の一時パラメータを取得する
                AngleParameterItem angleParameterItem2 = (AngleParameterItem)command.GetTemporaryParameterItemByID(3);
                if (angleParameterItem2 != null && !angleParameterItem2.IsEmpty && angleParameterItem2.Group.CurrentItem.ID == 3)
                    angle = (180.0 - angleParameterItem2.Angle) / 2;
            }

            // 方向点の一時パラメータを取得する
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetTemporaryParameterItemByID(4);
            if(pointParameterItem == null || pointParameterItem.IsEmpty)
                return;

            Point2d point = pointParameterItem.Point;
            // 現在のレイアウトの座標系からワールド座標に変換
            point = ActiveDocument.CurrentLayout.TransformPointLocalToWorld(point);
            // ワールド座標から図形の座標系に変換
            point = shapeParameterItem.Shape.Layout.TransformPointWorldToLocal(point);
            // 二等辺三角形の二辺の線図形を取得
            LineShape[] lineShapes = Create2Sides(lineShape, length, angle, point);

            try
            {
                if(lineShapes.Length == 2)
                {
                    // ラバーバンドの色を設定
                    lineShapes[0].ColorNumber = 3;
                    lineShapes[1].ColorNumber = 3;

                    // 現在の座標系に変換してラバーバンド表示
                    Shape[] lineShapes2 = shapeParameterItem.Shape.Layout.TransformShapesLocalToWorld(lineShapes);
                    Shape[] lineShapes3 = ActiveDocument.CurrentLayout.TransformShapesWorldToLocal(lineShapes2);
                    command.DrawShapeRubberBand(lineShapes3);
                }
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// 二等辺三角形の二辺を作成する
        /// </summary>
        /// <param name="shape"></param>
        /// <param name="length"></param>
        /// <param name="angle"></param>
        /// <param name="point"></param>
        private LineShape[] Create2Sides(LineShape shape, double length, double angle, Point2d point)
        {
            ArrayList shapes = new ArrayList();

            // 底辺の長さ
            double baseLineLength = shape.Length;

            // 長さが指定されていない場合は低角から長さを計算する
            if(length == 0)
            {
                if(angle < 1.0 || angle > 89.0)
                    return (LineShape[])shapes.ToArray(typeof(LineShape));
                length = (baseLineLength / 2) / Math.Cos(Geometry.DegreeToRadian(angle));
            }

            // 二辺の長さが足りない場合
            if(baseLineLength >= length * 2)
                return (LineShape[])shapes.ToArray(typeof(LineShape));

            // 方向点がどちら側かを取得
            PointSide pointSide = shape.GetPointSide(point);

            if(pointSide == PointSide.On)
                return (LineShape[])shapes.ToArray(typeof(LineShape));

            // 底辺の角度
            double baseLineAngle = Geometry.GetAngle(shape.StartPoint, shape.EndPoint);

            if(pointSide == PointSide.Right)
                baseLineAngle = Geometry.WrapAngleTo360(baseLineAngle + 180.0);

            // 二等辺三角形の高さを求める
            double triangleHeight = Math.Sqrt(Math.Pow(length, 2) - Math.Pow(baseLineLength / 2, 2));

            // 底辺を水平としたときの底辺の中点を求める
            Point2d horizontalBaseCenter =
                Geometry.GetMiddlePoint(
                    shape.StartPoint,
                    Geometry.RotatePoint(shape.EndPoint, shape.StartPoint, Geometry.WrapAngleTo360(-baseLineAngle)));

            // 中点のY座標に高さを足す
            Point2d horizontalApex = horizontalBaseCenter;
            horizontalApex.Y += triangleHeight;

            // 元の角度に戻す
            Point2d apexPoint = Geometry.RotatePoint(horizontalApex, shape.StartPoint, Geometry.WrapAngleTo360(baseLineAngle));

            LineShape line1 = (LineShape)shape.Clone();
            line1.SetPoint(shape.StartPoint, apexPoint);
            shapes.Add(line1);

            LineShape line2 = (LineShape)shape.Clone();
            line2.SetPoint(shape.EndPoint, apexPoint);
            shapes.Add(line2);

            return (LineShape[])shapes.ToArray(typeof(LineShape));
        }

        private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
		{
			AssemblyName assemblyName = new AssemblyName(args.Name);

			foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
			{
				if(AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
					return assembly;
			}

			if(assemblyName.Name.EndsWith(".resources"))
			{
				if(assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
				{
					string dllPath = Path.GetDirectoryName(this.AddInFullPath);
					dllPath = Path.Combine(dllPath, assemblyName.CultureName);
					dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

					if(File.Exists(dllPath))
					{
						try
						{
							return Assembly.LoadFrom(dllPath);
						}
						catch
						{
						}
					}
				}
			}

			return null;
		}
	}
}
